unit main;

interface

uses
	Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
	StdCtrls, ExtCtrls, ComCtrls, Registry, Stringz, FileCtrl, ShlObj;

type
	TConversion = Record
		Name : String;
		ValueCount : Integer;
		ConversionValues : String;
		TEDValues : String;
	End;

	TFormMain = class(TForm)
		lblReplaySpeed: TLabel;
		lblFlashColor: TLabel;
		lblColor: TLabel;
		LabelConversion: TLabel;
		LabelBits: TLabel;
		lblSaveAs: TLabel;
		LabelInputFiles: TLabel;
		lblDest: TLabel;

		Panel1: TPanel;
		Panel2: TPanel;
		Panel3: TPanel;
		Panel4: TPanel;
		OpenDialog1: TOpenDialog;
		Image1: TImage;
		EditStep: TEdit;
		EditInputFiles: TEdit;
		EditDestFolder: TEdit;
		EditReplaySpeed: TEdit;
		EditFlashColor: TEdit;
		EditReplaySpeedHex: TEdit;
		EditReplayFreq: TEdit;
		UpDown1: TUpDown;
		UpDownSpeed: TUpDown;
		UpDownColor: TUpDown;
		memoDump: TMemo;
		chkSigned: TCheckBox;
		chk1File: TCheckBox;
		optSaveExec: TRadioButton;
		optSaveData: TRadioButton;
		optSaveSource: TRadioButton;
		cboConversion: TComboBox;
		optBits1: TRadioButton;
		optBits2: TRadioButton;
		optBits4: TRadioButton;
		optBits8: TRadioButton;
		optKeep: TRadioButton;
		optSet: TRadioButton;
		cmdDraw: TButton;
		cmdOpen: TButton;
		cmdView: TButton;
		cmdBrowse: TButton;
		cmdBrowse2: TButton;

		procedure cmdBrowseClick(Sender: TObject);
		procedure cmdViewClick(Sender: TObject);
		procedure memoDumpDblClick(Sender: TObject);
		procedure Image1DblClick(Sender: TObject);
		procedure cmdDrawClick(Sender: TObject);
		procedure FormCreate(Sender: TObject);
		procedure cmdOpenClick(Sender: TObject);
		procedure EditInputFilesChange(Sender: TObject);
		procedure EditFlashColorChange(Sender: TObject);
		procedure EditReplaySpeedChange(Sender: TObject);
		procedure FormDestroy(Sender: TObject);
		procedure cboConversionChange(Sender: TObject);
		procedure FormResize(Sender: TObject);
		procedure EnableDisableOptionsClick(Sender: TObject);
    procedure EditReplayFreqKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure cmdBrowse2Click(Sender: TObject);
	private
		{ Private declarations }
		nConversions : Integer; // total number of conversions
		iConversion : Integer; // the currently selected conversion
		Conversions : Array [ 1 .. 100 ] Of TConversion;
		ByteConv : Array [ 0 .. 255 ] Of Byte;
		f, fo : File;
		iBits : Integer;

		procedure InitConversions;
		procedure InitConv ( iNewConversion : Integer );
		procedure Dump ( s : String );
		Function Commas ( s : String ) : String;
		procedure ConvertFile ( s : String );
		Function ForceExtension ( s, Ext : String ) : String;
		function FreqToRate( freq : Integer ) : Byte;
	public
		{ Public declarations }
		procedure Settings ( bSave : Boolean );
	end;

var
	FormMain: TFormMain;
	sFolderInit : String; // for BrowseForFolder

implementation

{$R *.DFM}

Function TFormMain.ForceExtension ( s, Ext : String ) : String;
var
	i : Integer;
begin
	i := PosR ( '.', s );
	If ( i > Length ( s ) - 5 ) Then
	Begin
		s := Copy ( s, 1, i ) + Ext;
	End;
	ForceExtension := s;
End;

procedure TFormMain.Dump ( s : String );
Begin
	If Image1.Visible Then
	Begin
		Image1.Visible := False;
		memoDump.Visible := True;
	End;
	//
	memoDump.Lines.Add ( s );
End;

procedure TFormMain.cmdBrowseClick(Sender: TObject);
begin
	With OpenDialog1 Do
	Begin
		Title := 'Select WAV file to convert';
		//InitialDir := 'C:\SAMPLES\DRUMS\';
		If Execute Then
		Begin
			EditInputFiles.Text := FileName;
		End;
	End;
end;

procedure TFormMain.ConvertFile ( s : String );

const
	MaxSize = $ED00 * 2;
	ChunkSize = $8000;

Type
	TData = Array [ 0 .. ChunkSize - 1 ] Of Byte;
	PData = ^TData;

var
	OutSize, OutBlock : Integer;
	l, l2 : Longint;
	//av : Longint;
	d_in : Array [ 0 .. 31 ] Of PData;
	d_out : Array [ 0 .. 31 ] Of PData;
	x : Byte;

	Sign1, Sign2 : String;

	ByteUsed : Array [ 0 .. 255 ] Of Boolean;
	EOFByte : Byte;

	// for Wav stuff
	lSampleRate : Longint;
	lLen : Longint;
	lBits : Word;

// Allocates and reads "l" bytes
procedure ReadData;
var
	l0, l1, d0 : Longint;
begin
	l0 := l;
	d0 := 0;
	While l0 > 0 Do
	Begin
		If l0 > ChunkSize Then
			l1 := ChunkSize
		Else
			l1 := l0;

		GetMem ( d_in [ d0 ], l1 );
		BlockRead ( f, d_in [ d0 ]^, l1 );
		Dec ( l0, l1 );
		Inc ( d0 );
	End;
	CloseFile ( f );
end;

// PutData
procedure PutData ( b0 : Byte );
Begin
	If ( OutSize Mod ChunkSize ) = 0 Then
	Begin
		//
		// allocate and start new block
		Inc ( OutBlock );
		GetMem ( d_out [ OutBlock ], ChunkSize );
	End;
	d_out [ OutBlock ]^ [ OutSize Mod ChunkSize ] := b0;
	Inc ( OutSize );
End;

procedure PutString ( s : String );
Var
	i : Integer;
begin
	For i := 1 To Length ( s ) Do
	begin
		PutData ( Ord ( s [ i ] ) );
	end;
End;

// GetData
function GetData ( l0 : Longint ) : Byte;
Var
	d0 : Byte;
Begin
	d0 := 0;
	While l0 >= ChunkSize Do
	Begin
		Dec ( l0, ChunkSize );
		Inc ( d0 );
	End;
	GetData := d_in [ d0 ]^ [ l0 ];
End;

procedure WriteNewFile;
var
	s0, sDest : String;
	i : Integer;
begin
	s0 := s;
	i := Length ( s0 );
	While s0 [ i ] <> '\' Do
		dec ( i );
	s0 := Copy ( s0, i + 1, Length ( s0 ) - i );

	sDest := EditDestFolder.Text;

	If chk1File.Checked Then
	Begin
		If FileExists ( sDest ) Then
		Begin
			AssignFile ( fo, sDest );
			Reset ( fo, 1 );
			Seek ( fo, FileSize ( fo ) );
			Dump ( 'Writing to ' + sDest + ' (append).' );
		End
		Else
		Begin
			AssignFile ( fo, sDest );
			ReWrite ( fo, 1 );
			Dump ( 'Writing to ' + sDest + ' (new file).' );
		End;
	End
	Else
	Begin
		If optSaveExec.Checked Then
		Begin
			sDest := ForceExtension ( sDest + s0, 'prg' );
		End
		Else
		Begin
			If optSaveSource.Checked Then
			Begin
				sDest := ForceExtension ( sDest + s0, 'asm' );
			End
			Else
			Begin
				sDest := sDest + s0;
			End;
		End;
		//
		AssignFile ( fo, sDest );
		ReWrite ( fo, 1 );
		Dump ( 'Opened ' + sDest + ' (new file).' );
	End;
end;

procedure WriteHeader;
Var
	LocalDataPointer : Integer;
	LocalData : Array [ 0 .. 1024 ] Of Byte;
	i, x : Integer;
	TEDValuesPointer : Word;
	ColorsPointer : Word;
	DataPointer : Word;

Procedure SendByte ( b : Byte );
Begin
	LocalData [ LocalDataPointer ] := b;
	Inc ( LocalDataPointer );
End;

Procedure SendBytes ( s : String );
Var
	i : Integer;
	Bytes : TStringList;
Begin
	If KeepFromRight ( s, 1 ) <> ',' Then s := s + ',';
	Bytes := Split ( s, ',' );
	For i := 0 To Bytes.Count - 1 Do
	Begin
		SendByte ( SafeVal ( Bytes [ i ] ) );
	End;
End;

Procedure SendRelativeJump ( DestAddr : Word );
Begin
	SendByte ( 254 - ( ( LocalDataPointer + $1000 - 2 ) - DestAddr ) );
End;

begin
	//
	// --- Write header only if "Executable" option is selected
	//
	If optSaveExec.Checked Then
	Begin
		LocalDataPointer := 0;
		TEDValuesPointer := 0;

		// These are checked "by hand":
		// it's where the TED lookup values will go after the code
		Case iBits Of

			1 :
			Begin
				TEDValuesPointer := $1127;
			End;

			2 :
			Begin
				TEDValuesPointer := $10AB;
			End;

			4 :
			Begin
				TEDValuesPointer := $106D;
			End;

			8 :
			Begin
				TEDValuesPointer := $1065;
			End;
		End;
		ColorsPointer := TEDValuesPointer + Conversions [ iConversion ].ValueCount;
		DataPointer := ColorsPointer + Conversions [ iConversion ].ValueCount;

		SendBytes ( '$01,$10, $0B,$10, $00,$00' );
		SendBytes ( '$9E,$34,$31,$31,$30,$00,$00,$00,' );

		SendByte ( EOFByte ); // eof byte

		SendBytes ( '$78,' );
		SendBytes ( '$8D,$3F,$FF,' );
		SendBytes ( '$A9,' );
		If( optKeep.Checked ) Then
			SendByte( FreqToRate( lSampleRate ) )
		Else
			SendByte ( SafeVal ( EditReplaySpeed.Text ) );
		SendBytes ( '$A0,$00,' );
		SendBytes ( '$8C,$06,$FF,' );
		SendBytes ( '$8D,$00,$FF,' );
		SendBytes ( '$8C,$01,$FF,' );
		SendBytes ( '$A9,' );
		SendByte ( DataPointer And 255 );
		SendBytes ( '$85,$C0,' );
		SendBytes ( '$A9,' );
		SendByte ( DataPointer Shr 8 );
		SendBytes ( '$85,$C1,' );

		If ( iBits = 8 ) Then
		Begin
			// for quasi 8 bit digi
			// the two ted channels have to be set up
			SendBytes ( '$A9,$FD,$8D,$0E,$FF,$A9,$FE,$8D,$0F,$FF,' );
			SendBytes ( '$A9,$03,$8D,$10,$FF,$AD,$12,$FF,$09,$03,$8D,$12,$FF' );
		End;

		For i := 0 To ( 8 Div iBits ) - 1 Do
		Begin
			SendBytes ( '$A9,$08,$2C,$09,$FF,$F0,$FB,$8D,$09,$FF,' );
			SendBytes ( '$B1,$C0,' );

			If i = 0 Then
			Begin
				SendBytes ( '$CD,$0D,$10' ); // cmp $100D
				SendBytes ( '$F0,' );
				SendRelativeJump ( $101F );
			End;

			Case iBits Of

				1 :
				Begin
					For x := 0 To i - 1 Do
					Begin
						SendByte ( $4A );
					End;
					If ( i < ( 8 Div iBits ) - 1 ) Then
						SendBytes ( '$29,$01,' );
				End;

				2 :
				Begin
					For x := 0 To i - 1 Do
					Begin
						SendBytes ( '$4A,$4A,' );
					End;
					If ( i < ( 8 Div iBits ) - 1 ) Then
						SendBytes ( '$29,$03,' );
				End;

				4 :
				Begin
					If i = 0 Then
						SendBytes ( '$29,$0F,' ) // and #$0F
					Else
						SendBytes ( '$4A,$4A,$4A,$4A,' ); // 4 lsr
				End;

				8 :
				Begin
					// no processing, use the whole byte as is :-)
				End;
			End;

			SendBytes ( '$AA,' );
			SendBytes ( '$BD,' );
			SendByte ( TEDValuesPointer And 255 );
			SendByte ( TEDValuesPointer Shr 8 );
			SendBytes ( '$8D,$11,$FF,' );
			SendBytes ( '$BD,' );
			SendByte ( ColorsPointer And 255 );
			SendByte ( ColorsPointer Shr 8 );
			SendBytes ( '$8D,$19,$FF,' );
		End;

		SendBytes ( '$E6,$C0,' );
		SendBytes ( '$D0,' );
		//SendRelativeJump ( $1027 );
		SendBytes ( '$02,' );
		SendBytes ( '$E6,$C1,' );
		SendBytes ( '$4C,$27,$10,' );
		//
		SendBytes ( Conversions [ iConversion ].TEDValues );
		For i := 0 To Conversions [ iConversion ].ValueCount - 1 Do
		Begin
			SendByte ( Trunc ( ( i / Conversions [ iConversion ].ValueCount ) * 7 ) Shl 4 Or UpDownColor.Position );
		End;
		//
		BlockWrite ( fo, LocalData, LocalDataPointer );
	End;
end;

procedure WriteData;
var
	l1 : Longint;
begin
	OutBlock := 0;
	//
	While OutSize > 0 Do
	Begin
		//
		l1 := OutSize;
		If l1 > ChunkSize Then
			l1 := ChunkSize;
		//
		BlockWrite ( fo, d_out [ OutBlock ]^, l1 );
		FreeMem ( d_out [ OutBlock ], ChunkSize );
		Inc ( OutBlock );
		//
		Dec ( OutSize, l1 );
	End;

	If optSaveExec.Checked Then
	begin
		// Write end marker.
		BlockWrite ( fo, EOFByte, 1 );
	end;

	CloseFile ( fo );
end;

function ConvertByte ( b1 : Byte ) : Byte;
Begin
	ConvertByte := ByteConv [ b1 ];
End;

procedure DoConv;
var
	i : Longint;
	b : Byte;
Begin
	// determine number of bits used
	If optBits1.Checked Then iBits := 1;
	If optBits2.Checked Then iBits := 2;
	If optBits4.Checked Then iBits := 4;
	If optBits8.Checked Then iBits := 8;

	// clear all bytes used flag
	// this array keeps track of what bytes have been outputted
	// the converter will try to find one byte that's NOT used
	// and use that as an EOF byte in the player
	For b := 0 To 255 Do
	Begin
		ByteUsed [ b ] := False;
	End;

	ReadData;
	WriteNewFile;

	// This code was writte to try and decide if wave is signed
	// but it's not reliable
	{
	x := 0;

	av := 0;
	For i := 1 To l - 1 Do
	Begin
		If ( GetData ( i - 1 ) < 16 )
		And ( GetData ( i ) > 255 - 16 ) Then
		Begin
			Inc ( av );
		End;
	End;
	Dump ( 'Jumps: ' + IntToStr ( av ) );

	If av >= 80 Then
	Begin
		x := 128;
		Dump ( 'Using XOR' );
	End
	Else
	Begin
		Dump ( 'Normal' );
	End;
	}

	// End byte
	l2 := l;
	If ( iBits = 4 ) Then
	Begin
		// if 4 bits, cut off trailing last byte
		If ( l And 1 ) = 0 Then Dec ( l2 );
	End;

	i := 0;
	//
	OutSize := 0;
	OutBlock := -1;
	//
	While ( i < l2 ) Do
	Begin
		Case iBits Of
			1 : b := ConvertByte ( GetData ( i ) Xor x ) Or
							 ConvertByte ( GetData ( i + 1 ) Xor x ) Shl 1 Or
							 ConvertByte ( GetData ( i + 2 ) Xor x ) Shl 2 Or
							 ConvertByte ( GetData ( i + 3 ) Xor x ) Shl 3 Or
							 ConvertByte ( GetData ( i + 4 ) Xor x ) Shl 4 Or
							 ConvertByte ( GetData ( i + 5 ) Xor x ) Shl 5 Or
							 ConvertByte ( GetData ( i + 6 ) Xor x ) Shl 6 Or
							 ConvertByte ( GetData ( i + 7 ) Xor x ) Shl 7;

			2 : b := ConvertByte ( GetData ( i ) Xor x ) Or
							 ConvertByte ( GetData ( i + 1 ) Xor x ) Shl 2 Or
							 ConvertByte ( GetData ( i + 2 ) Xor x ) Shl 4 Or
							 ConvertByte ( GetData ( i + 4 ) Xor x ) Shl 6;

			4 : b := ConvertByte ( GetData ( i ) Xor x ) Or
							 ConvertByte ( GetData ( i + 1 ) Xor x ) Shl 4;

			Else b := ConvertByte ( GetData ( i ) Xor x );
		End;
		//
		// Flag that this byte has been used
		ByteUsed [ b ] := True;
		//
		If optSaveSource.Checked Then
		Begin
			//
			// send byte as text
			//
			If ( i And 31 ) = 0 Then
			Begin
				If ( ( i And 511 ) = 0 ) And ( i > 0 ) Then
				Begin
					PutString ( #13 + #10 );
				End;
				PutString ( #13 + #10 + #9 + 'DB ' );
			End
			Else
			Begin
				PutString ( ',' );
			End;
			PutString ( '$' + IntToHex ( b, 2 ) );
		End
		Else
		Begin
			//
			// send just the byte
			//
			PutData ( b );
		End;
		//
		Case iBits Of
			1 : Inc ( i, 8 );
			2 : Inc ( i, 4 );
			4 : Inc ( i, 2 );
			Else Inc ( i );
		End;
	End;
	//
	i := -2;
	b := 255;
	While ( i = -2 ) Do
	Begin
		If Not ByteUsed [ b ] Then i := b; // found one not used
		Dec ( b );
		If ( b = 255 ) Then Inc ( i ); // exit from loop
	End;
	If i > 0 Then
		Dump ( 'EOF byte: ' + IntToStr ( i ) )
	Else
		Dump ( '*** ERROR: Could not find an EOF byte. Default player won''t work properly.' );
	EOFByte := i;
	//
	If optSaveSource.Checked Then
	Begin
		PutString ( #13 + #10 );
	End;
	//
	WriteHeader;
	WriteData;

	Dump ( 'Finished.' );
	Dump ( '' );
End;

begin
	// Get filename
	If FileExists ( s ) Then
	Begin
		AssignFile ( f, s );
		Reset ( f, 1 );

		Dump ( 'Opened file ' + s );

		l := FileSize ( f );

		Dump ( 'File size  : ' + Commas ( IntToStr ( l ) ) );

		If l < $100 Then
		Begin

			Dump ( 'File is too small, conversion cancelled.' );
			CloseFile ( f );

		End
		Else
		Begin

			Sign1 := '****';
			Sign2 := '****';

			BlockRead ( f, Sign1 [ 1 ], 4 );
			Seek ( f, $50 - 4 );
			BlockRead ( f, Sign2 [ 1 ], 4 );

			If ( Sign1 = 'IMPS' ) Or ( Sign2 = 'SCRS' ) Then
			Begin

				If ( Sign1 = 'IMPS' ) Then
				Begin
					Dump ( 'File format: Impulse Tracker Sample' );
					x := 128;
				End
				Else
					Dump ( 'File format: Scream Tracker Sample' );

				l := l - $50;

				Seek ( f, $50 );

				If ( l > MaxSize ) Then
				Begin
					l := MaxSize;
					Dump ( 'File too long, data reduced to ' +
						Commas ( IntToStr ( l ) ) + ' bytes.' );
				End;

				DoConv;
			End
			Else
			Begin

				If Sign1 = 'RIFF' Then
				Begin
					Seek( f, $18 );
					BlockRead( f, lSampleRate, 4 );
					Dump ( 'File format: Wave (' + Comma( lSampleRate ) + ' Hz)' );
					Seek ( f, $20 );
					BlockRead ( f, lBits, 2 );
					Seek ( f, $24 );
					BlockRead ( f, Sign1 [ 1 ], 4 );
					If ( Sign1 <> 'data' )
					Or ( ( lBits <> 1 ) And ( lBits <> 2 ) ) Then
					Begin
						Dump ( 'Unrecognized format. Sorry... :-(' );
						CloseFile ( f );
					End
					Else
					Begin
						If lBits = 2 Then
						Begin
							//Dump ( '16 bit samples are currently not supported...' );
							//CloseFile ( f );

							BlockRead ( f, lLen, 4 );
							l := lLen;
							If ( l > MaxSize ) Then
							Begin
								l := MaxSize;
								Dump ( 'File too long, data reduced to ' +
									Commas ( IntToStr ( l ) ) + ' bytes.' );
							End;

							DoConv;

						End
						Else
						Begin
							BlockRead ( f, lLen, 4 );
							l := lLen;
							If ( l > MaxSize ) Then
							Begin
								l := MaxSize;
								Dump ( 'File too long, data reduced to ' +
									Commas ( IntToStr ( l ) ) + ' bytes.' );
							End;

							DoConv;
						End;
					End;
				End
				Else
				Begin
					// raw?
					Dump ( 'Raw file' );

					Seek ( f, 0 );

					If ( l > MaxSize ) Then
					Begin
						l := MaxSize;
						Dump ( 'File too long, data reduced to ' +
							Commas ( IntToStr ( l ) ) + ' bytes.' );
					End;

					DoConv;
				End;
			End;
		End;
	End;
end;

procedure TFormMain.cmdViewClick(Sender: TObject);
const
	MaxSize = $FF00;
Type
	TData = Array [ 0 .. MaxSize - 1 ] Of Byte;
	PData = ^TData;
var
	f : File;
	s : String;
	i, l : Longint;
	d : PData;
	step, p : Longint;
	x : Byte;
	Resize : Double;
begin
	s := EditInputFiles.Text;
	If FileExists ( s ) Then
	Begin
		AssignFile ( f, s );
		Reset ( f, 1 );
		//
		memoDump.Visible := False;
		Image1.Visible := True;
		//
		Resize := Image1.Height / 256;
		//
		l := FileSize ( f );
		l := l - $50;
		If l > 0 Then
		Begin
			Seek ( f, $50 );
			If ( l > MaxSize ) Then
				l := MaxSize;

			GetMem ( d, l );
			BlockRead ( f, d^, l );
			CloseFile ( f );

			If chkSigned.Checked Then
				x := 128
			Else
				x := 0;

			Image1.Canvas.FillRect ( Image1.Canvas.ClipRect );

			Image1.Canvas.Pen.Color := clBlue;
			Image1.Canvas.MoveTo ( 0, Round ( 128 * Resize ) );
			Image1.Canvas.LineTo ( Image1.Width + 1, Round ( 128 * Resize ) );

			Step := UpDown1.Position;

			i := 0;
			p := 0;

			Image1.Canvas.Pen.Color := clBlack;
			Image1.Canvas.MoveTo ( 0, Round ( 128 * Resize ) );

			While i < Image1.Width - 1 Do
			Begin
				Image1.Canvas.LineTo ( i, Round ( ( d^ [ p ] Xor x ) * Resize ) );
				Inc ( i );
				Inc ( p, step );
			End;

			FreeMem ( d, l );
		End
		Else
		Begin
			CloseFile ( f );
		End;
	End;
end;

Function TFormMain.Commas ( s : String ) : String;
var
	i : Integer;
begin
	i := Length ( s ) - 3;
	While i > 0 Do
	Begin
		If s [ i ] = ' ' Then
			s := Copy ( s, 1, i ) + ' ' + Copy ( s, i + 1, Length ( s ) - i )
		Else
			s := Copy ( s, 1, i ) + ',' + Copy ( s, i + 1, Length ( s ) - i );
		Dec ( i, 3 );
	End;
	Commas := s;
end;

procedure TFormMain.memoDumpDblClick(Sender: TObject);
begin
	memoDump.Visible := False;
	Image1.Visible := True;
end;

procedure TFormMain.Image1DblClick(Sender: TObject);
begin
	memoDump.Visible := True;
end;

procedure TFormMain.InitConversions;
var
	t : TextFile;
	sPath : String;
	s : String;
Begin
	nConversions := 0;
	sPath := Copy ( Application.ExeName, 1, PosR ( '\', sPath ) ) + 'convert.ini';
	//
	AssignFile ( t, sPath );
	Reset ( t );
	//
	While Not Eof ( t ) Do
	Begin
		ReadLn ( t, s );
		If Trim ( s ) <> '' Then
		Begin
			Inc ( nConversions );
			Conversions [ nConversions ].Name := s;
			ReadLn ( t, s );
			Conversions [ nConversions ].ConversionValues := s;
			Conversions [ nConversions ].ValueCount := Split ( s, ';' ).Count;
			ReadLn ( t, s );
			Conversions [ nConversions ].TEDValues := s;
			//
			cboConversion.Items.Add ( Conversions [ nConversions ].Name );
		End;
	End;
	//
	cboConversion.ItemIndex := 0;
	//
	CloseFile ( t );
end;

procedure TFormMain.InitConv ( iNewConversion : Integer );
Var
	i, j, b, cPos : Integer;
	nValues : Integer;
	s, sEval : String;
begin
	b := 0;
	i := 0;
	nValues := 0;
	//
	iConversion := iNewConversion;
	//
	s := Trim ( Conversions [ iConversion ].ConversionValues );
	If ( KeepFromRight ( s, 1 ) <> ';' ) Then
		s := s + ';';
	//s := '3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;' + '3;3;4;3;3;' +
	//	'3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;3;';
	//s := '15;14;15;14;10;09;09;10;09;09;28;14;14;28;29;29;';
	//s := '30;35;35;56;35;35;30;';
	//s := '32;32;32;32;32;32;32;32;';
	//
	While s <> '' Do
	Begin
		Inc ( nValues );
		cPos := Pos ( ';', s );
		j := 0;
		sEval := Copy ( s, 1, cPos - 1 );
		Try
			j := StrToInt ( sEval );
		Except
			ShowMessage ( 'Could not evaluate: "' + sEval + '"' );
		End;
		//
		While j > 0 Do
		Begin
			ByteConv [ i ] := b;
			Inc ( i );
			Dec ( j );
		End;
		//
		Inc ( b );
		s := RemoveFromLeft ( s, cPos );
	End;
	//
	Dump ( 'Using conversion #' + IntToStr ( iConversion ) + ': ' + IntToStr ( nValues ) + ' values' );
	//
	If nValues > 16 Then
	Begin
		optBits1.Enabled := False;
		optBits2.Enabled := False;
		optBits4.Enabled := False;
		optBits8.Checked := True;
	End
	Else
	Begin
		If nValues > 4 Then
		Begin
			optBits1.Enabled := False;
			optBits2.Enabled := False;
			optBits4.Enabled := True;
			optBits4.Checked := True;
		End
		Else
		Begin
			If nValues > 2 Then
			Begin
				optBits1.Enabled := False;
				optBits2.Enabled := True;
				optBits2.Checked := True;
				optBits4.Enabled := True;
			End
			Else
			Begin
				optBits1.Enabled := True;
				optBits1.Checked := True;
				optBits2.Enabled := True;
				optBits4.Enabled := True;
			End;
		End;
	End;
End;

procedure TFormMain.cmdDrawClick(Sender: TObject);
var
	i : Integer;
	y : Byte;
begin
	memoDump.Visible := False;
	Image1.Visible := True;
	//
	For i := 0 To 255 Do
	Begin
		y := ByteConv [ i ] + 1;
		Image1.Canvas.Pen.Color := RGB ( y * 10, y * 20, y * 30 );
		Image1.Canvas.MoveTo ( i, y * 10 );
		Image1.Canvas.LineTo ( i, y );

		If ( i And 15 ) = 0 Then
		Begin
			If ( i And 63 ) = 0 Then
				Image1.Canvas.Pen.Color := RGB ( 255, 0, 0 )
			Else
				Image1.Canvas.Pen.Color := RGB ( 0, 0, 255 );

			Image1.Canvas.MoveTo ( i, 0 );
			Image1.Canvas.LineTo ( i, 100 );
		End;
	End;
end;

procedure TFormMain.FormCreate(Sender: TObject);
begin
	Settings ( False );
	//
	InitConversions;
	InitConv ( 1 );
	//
	EditInputFilesChange(Sender);
	//
	EnableDisableOptionsClick(Sender);
end;

procedure TFormMain.cmdOpenClick(Sender: TObject);
var
	s, sPath : String;
	sr : TSearchRec;
	i : Integer;
	Count : Integer;
begin
	s := EditInputFiles.Text;

	If ( Pos ( '*', s ) > 0 )
	Or ( Pos ( '?', s ) > 0 ) Then
	Begin
		i := Length ( s );
		While s [ i ] <> '\' Do
			dec ( i );
		sPath := Copy ( s, 1, i );

		Count := 0;
		i := FindFirst ( s, faAnyFile, sr );
		While i = 0 Do
		Begin
			Inc ( Count );
			ConvertFile ( sPath + sr.Name );
			i := FindNext ( sr );
		End;
		//
		Dump ( IntToStr ( Count ) + ' file(s) processed.' );
	End
	Else
	Begin
		ConvertFile ( s );
	End;
end;

procedure TFormMain.EditInputFilesChange(Sender: TObject);
begin
	cmdOpen.Enabled := ( EditInputFiles.Text <> '' ) And ( EditDestFolder.Text <> '' );
	cmdView.Enabled := cmdOpen.Enabled;
end;

procedure TFormMain.EditFlashColorChange(Sender: TObject);
Var
	s : String;
begin
	Case UpDownColor.Position Of
		0 : s := 'Black';
		1 : s := 'White';
		2 : s := 'Red';
		3 : s := 'Cyan';
		4 : s := 'Purple';
		5 : s := 'Green';
		6 : s := 'Blue';
		7 : s := 'Yellow';
		8 : s := 'Orange';
		9 : s := 'Brown';
		10 : s := 'Yellow-Green';
		11 : s := 'Pink';
		12 : s := 'Blue-Green';
		13 : s := 'Light Blue';
		14 : s := 'Dark Blue';
		15 : s := 'Light Green';
	End;
	//
	lblColor.Caption := '(' + s + ')';
end;

procedure TFormMain.EditReplaySpeedChange(Sender: TObject);
begin
	// PAL  17.734470 / 20
	// NTSC 14.318180 / 16

	EditReplaySpeedHex.Text := '$' + IntToHex ( UpDownSpeed.Position, 2 );
	EditReplayFreq.Text := IntToStr ( Round ( ( 17734470 / 20 ) / UpDownSpeed.Position ) );
end;

function TFormMain.FreqToRate( freq : Integer ) : Byte;
Begin
	FreqToRate := Round( ( 17734470 / 20 ) / freq );
End;

procedure TFormMain.EditReplayFreqKeyUp(Sender: TObject; var Key: Word;
	Shift: TShiftState);
begin
	//UpDownSpeed.Position := FreqToRate( SafeVal( EditReplayFreq.Text ) );
end;

procedure TFormMain.Settings ( bSave : Boolean );
var
	r : TRegistry;

Function GetBool ( s : String ) : Boolean;
Begin
	If r.ValueExists ( s ) Then
		GetBool := r.ReadBool ( s )
	Else
		GetBool := False;
End;

begin
	r := TRegistry.Create;
	r.RootKey := HKEY_CURRENT_USER;
	If r.OpenKey ( 'Software\Csabo\WavConv', bSave ) Then
	Begin
		If bSave Then
		Begin
			// Write settings to registry
			r.WriteString ( 'InputFiles', EditInputFiles.Text );
			r.WriteString ( 'DestFolder', EditDestFolder.Text );
			r.WriteBool ( 'optSaveExec.Checked', optSaveExec.Checked );
			r.WriteBool ( 'optSaveData.Checked', optSaveData.Checked );
			r.WriteBool ( 'optSaveSource.Checked', optSaveSource.Checked );
			r.WriteBool ( 'optKeep.Checked', optKeep.Checked );
			r.WriteBool ( 'optSet.Checked', optSet.Checked );
		End
		Else
		Begin
			// Read settings from registry
			EditInputFiles.Text := r.ReadString ( 'InputFiles' );
			EditDestFolder.Text := r.ReadString ( 'DestFolder' );
			optSaveExec.Checked := GetBool ( 'optSaveExec.Checked' );
			optSaveData.Checked := GetBool ( 'optSaveData.Checked' );
			optSaveSource.Checked := GetBool ( 'optSaveSource.Checked' );
			optKeep.Checked := GetBool( 'optKeep.Checked' );
			optSet.Checked := GetBool( 'optSet.Checked' );
		End;
		r.CloseKey;
	End;
	r.Free;
end;

procedure TFormMain.FormDestroy(Sender: TObject);
begin
	Settings ( True );
end;

procedure TFormMain.cboConversionChange(Sender: TObject);
begin
	InitConv ( cboConversion.ItemIndex + 1 );
end;

procedure TFormMain.FormResize(Sender: TObject);
begin
	If Image1.Visible Then
	Begin
		Image1.Picture.Bitmap.Width := Image1.Width;
		Image1.Picture.Bitmap.Height := Image1.Height;
	End;
end;

procedure TFormMain.EnableDisableOptionsClick(Sender: TObject);
Begin
	optKeep.Enabled := optSaveExec.Checked;
	optSet.Enabled := optSaveExec.Checked;

	EditReplaySpeed.Enabled := optSaveExec.Checked And optSet.Checked;
	EditReplaySpeedHex.Enabled := optSaveExec.Checked And optSet.Checked;
	EditReplayFreq.Enabled := optSaveExec.Checked And optSet.Checked;
	UpDownSpeed.Enabled := optSaveExec.Checked And optSet.Checked;

	EditFlashColor.Enabled := optSaveExec.Checked;
	UpDownColor.Enabled := optSaveExec.Checked;

	If EditReplaySpeed.Enabled Then
		EditReplaySpeed.Color := clWindow
	Else
		EditReplaySpeed.Color := clBtnFace;

	If EditReplaySpeedHex.Enabled Then
		EditReplaySpeedHex.Color := clWindow
	Else
		EditReplaySpeedHex.Color := clBtnFace;

	If EditReplayFreq.Enabled Then
		EditReplayFreq.Color := clWindow
	Else
		EditReplayFreq.Color := clBtnFace;

	If EditFlashColor.Enabled Then
		EditFlashColor.Color := clWindow
	Else
		EditFlashColor.Color := clBtnFace;
End;

// Direct call to undocumented Windows function
Procedure FreePIDL( PIDL : pItemIDList); StdCall; External 'Shell32.DLL' Index 155;

Function BrowseCallback( Wnd : hWND; MessageID : uint; Param : LParam; Data : LParam ) : Integer StdCall;
Var
	Name : Array[ 0 .. MAX_PATH ] OF Char;
	pIDL : pItemIDList;
	s : String;
Begin
	Case MessageID OF
		BFFM_INITIALIZED:
		Begin
			If ( LENGTH( sFolderInit ) > 0 ) AND DirectoryExists( sFolderInit ) Then
			Begin
				SendMessage(Wnd, BFFM_SETSELECTION, Integer(TRUE), Integer( pChar(sFolderInit) ) );
			End;
		End;
	End;
	pIDL := Pointer(Param);
	s := '';
	IF Assigned(PIDL) THEN SHGetPathFromIDList(pIDL, Name);
	RESULT := 0;
End;

procedure TFormMain.cmdBrowse2Click( Sender: TObject );
VAR
	BrowseInfo : TBrowseInfo;
	ItemIDList : pItemIDList; // some would use PIDL here
	DisplayName : ARRAY[ 0 .. MAX_PATH ] OF CHAR;
begin
	BrowseInfo.hwndOwner := Handle;
	BrowseInfo.pidlRoot := NIL;
	BrowseInfo.pszDisplayName := @DisplayName[0];
	BrowseInfo.lpszTitle := 'Select Directory';
	BrowseInfo.ulFlags := BIF_RETURNONLYFSDIRS;
	BrowseInfo.lpfn := BrowseCallback;
	BrowseInfo.lParam := 0;
	BrowseInfo.iImage := 0;

	// Display browse folder set as the return value to itemlist
	sFolderInit := EditDestFolder.Text;
	ItemIDList := SHBrowseForFolder( BrowseInfo );
	Try
		// Get directory from the ItemIDList
		IF Assigned( ItemIDList ) THEN
		Begin
			IF SHGetPathFromIDList( ItemIDList, DisplayName ) Then
			Begin
				EditDestFolder.Text := DisplayName;
				If( KeepFromRight( EditDestFolder.Text, 1 ) <> '\' ) Then
				Begin
					EditDestFolder.Text := EditDestFolder.Text + '\';
				End;
			End;
		End;
	Finally
		FreePIDL( ItemIDList );
	End;
end;

end.
